# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::SQL::Mgmt::V2017_03_01_preview
  #
  # The Azure SQL Database management API provides a RESTful set of web
  # services that interact with Azure SQL Database services to manage your
  # databases. The API enables you to create, retrieve, update, and delete
  # databases.
  #
  class LongTermRetentionBackups
    include MsRestAzure

    #
    # Creates and initializes a new instance of the LongTermRetentionBackups class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [SqlManagementClient] reference to the SqlManagementClient
    attr_reader :client

    #
    # Gets a long term retention backup.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database.
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackup] operation results.
    #
    def get_by_resource_group(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      response = get_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a long term retention backup.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database.
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_by_resource_group_with_http_info(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      get_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
    end

    #
    # Gets a long term retention backup.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database.
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, 'long_term_retention_database_name is nil' if long_term_retention_database_name.nil?
      fail ArgumentError, 'backup_name is nil' if backup_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionDatabases/{longTermRetentionDatabaseName}/longTermRetentionBackups/{backupName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'longTermRetentionDatabaseName' => long_term_retention_database_name,'backupName' => backup_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackup.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Deletes a long term retention backup.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def delete_by_resource_group(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      response = delete_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      # Send request
      promise = begin_delete_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<LongTermRetentionBackup>] operation results.
    #
    def list_by_resource_group_database(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      first_page = list_by_resource_group_database_as_lazy(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_database_with_http_info(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      list_by_resource_group_database_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_database_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, 'long_term_retention_database_name is nil' if long_term_retention_database_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionDatabases/{longTermRetentionDatabaseName}/longTermRetentionBackups'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'longTermRetentionDatabaseName' => long_term_retention_database_name,'subscriptionId' => @client.subscription_id},
          query_params: {'onlyLatestPerDatabase' => only_latest_per_database,'databaseState' => database_state,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<LongTermRetentionBackup>] operation results.
    #
    def list_by_resource_group_location(resource_group_name, location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      first_page = list_by_resource_group_location_as_lazy(resource_group_name, location_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_location_with_http_info(resource_group_name, location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      list_by_resource_group_location_async(resource_group_name, location_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_location_async(resource_group_name, location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionBackups'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'locationName' => location_name,'subscriptionId' => @client.subscription_id},
          query_params: {'onlyLatestPerDatabase' => only_latest_per_database,'databaseState' => database_state,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<LongTermRetentionBackup>] operation results.
    #
    def list_by_resource_group_server(resource_group_name, location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      first_page = list_by_resource_group_server_as_lazy(resource_group_name, location_name, long_term_retention_server_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_server_with_http_info(resource_group_name, location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      list_by_resource_group_server_async(resource_group_name, location_name, long_term_retention_server_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_server_async(resource_group_name, location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionBackups'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'subscriptionId' => @client.subscription_id},
          query_params: {'onlyLatestPerDatabase' => only_latest_per_database,'databaseState' => database_state,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a long term retention backup.
    #
    # @param location_name [String] The location of the database.
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackup] operation results.
    #
    def get(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      response = get_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a long term retention backup.
    #
    # @param location_name [String] The location of the database.
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      get_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
    end

    #
    # Gets a long term retention backup.
    #
    # @param location_name [String] The location of the database.
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, 'long_term_retention_database_name is nil' if long_term_retention_database_name.nil?
      fail ArgumentError, 'backup_name is nil' if backup_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionDatabases/{longTermRetentionDatabaseName}/longTermRetentionBackups/{backupName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'longTermRetentionDatabaseName' => long_term_retention_database_name,'backupName' => backup_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackup.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Deletes a long term retention backup.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def delete(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      response = delete_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      # Send request
      promise = begin_delete_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<LongTermRetentionBackup>] operation results.
    #
    def list_by_database(location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      first_page = list_by_database_as_lazy(location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_database_with_http_info(location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      list_by_database_async(location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_database_async(location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, 'long_term_retention_database_name is nil' if long_term_retention_database_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionDatabases/{longTermRetentionDatabaseName}/longTermRetentionBackups'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'longTermRetentionDatabaseName' => long_term_retention_database_name,'subscriptionId' => @client.subscription_id},
          query_params: {'onlyLatestPerDatabase' => only_latest_per_database,'databaseState' => database_state,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<LongTermRetentionBackup>] operation results.
    #
    def list_by_location(location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      first_page = list_by_location_as_lazy(location_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_location_with_http_info(location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      list_by_location_async(location_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_location_async(location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionBackups'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'locationName' => location_name,'subscriptionId' => @client.subscription_id},
          query_params: {'onlyLatestPerDatabase' => only_latest_per_database,'databaseState' => database_state,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<LongTermRetentionBackup>] operation results.
    #
    def list_by_server(location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      first_page = list_by_server_as_lazy(location_name, long_term_retention_server_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_server_with_http_info(location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      list_by_server_async(location_name, long_term_retention_server_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_server_async(location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionBackups'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'subscriptionId' => @client.subscription_id},
          query_params: {'onlyLatestPerDatabase' => only_latest_per_database,'databaseState' => database_state,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Deletes a long term retention backup.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_delete_by_resource_group(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      response = begin_delete_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Deletes a long term retention backup.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_by_resource_group_with_http_info(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      begin_delete_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
    end

    #
    # Deletes a long term retention backup.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_by_resource_group_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, 'long_term_retention_database_name is nil' if long_term_retention_database_name.nil?
      fail ArgumentError, 'backup_name is nil' if backup_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionDatabases/{longTermRetentionDatabaseName}/longTermRetentionBackups/{backupName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'longTermRetentionDatabaseName' => long_term_retention_database_name,'backupName' => backup_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Deletes a long term retention backup.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_delete(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      response = begin_delete_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Deletes a long term retention backup.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_with_http_info(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      begin_delete_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:custom_headers).value!
    end

    #
    # Deletes a long term retention backup.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param backup_name [String] The backup name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_async(location_name, long_term_retention_server_name, long_term_retention_database_name, backup_name, custom_headers:nil)
      fail ArgumentError, 'location_name is nil' if location_name.nil?
      fail ArgumentError, 'long_term_retention_server_name is nil' if long_term_retention_server_name.nil?
      fail ArgumentError, 'long_term_retention_database_name is nil' if long_term_retention_database_name.nil?
      fail ArgumentError, 'backup_name is nil' if backup_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationName}/longTermRetentionServers/{longTermRetentionServerName}/longTermRetentionDatabases/{longTermRetentionDatabaseName}/longTermRetentionBackups/{backupName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'locationName' => location_name,'longTermRetentionServerName' => long_term_retention_server_name,'longTermRetentionDatabaseName' => long_term_retention_database_name,'backupName' => backup_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] operation results.
    #
    def list_by_resource_group_database_next(next_page_link, custom_headers:nil)
      response = list_by_resource_group_database_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_database_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_resource_group_database_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_database_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] operation results.
    #
    def list_by_resource_group_location_next(next_page_link, custom_headers:nil)
      response = list_by_resource_group_location_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_location_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_resource_group_location_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_location_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] operation results.
    #
    def list_by_resource_group_server_next(next_page_link, custom_headers:nil)
      response = list_by_resource_group_server_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_server_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_resource_group_server_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_server_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] operation results.
    #
    def list_by_database_next(next_page_link, custom_headers:nil)
      response = list_by_database_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_database_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_database_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_database_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] operation results.
    #
    def list_by_location_next(next_page_link, custom_headers:nil)
      response = list_by_location_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_location_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_location_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_location_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] operation results.
    #
    def list_by_server_next(next_page_link, custom_headers:nil)
      response = list_by_server_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_server_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_server_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_server_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_03_01_preview::Models::LongTermRetentionBackupListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] which provide lazy access to
    # pages of the response.
    #
    def list_by_resource_group_database_as_lazy(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      response = list_by_resource_group_database_async(resource_group_name, location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_resource_group_database_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] which provide lazy access to
    # pages of the response.
    #
    def list_by_resource_group_location_as_lazy(resource_group_name, location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      response = list_by_resource_group_location_async(resource_group_name, location_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_resource_group_location_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] which provide lazy access to
    # pages of the response.
    #
    def list_by_resource_group_server_as_lazy(resource_group_name, location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      response = list_by_resource_group_server_async(resource_group_name, location_name, long_term_retention_server_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_resource_group_server_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Lists all long term retention backups for a database.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param long_term_retention_database_name [String] The name of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] which provide lazy access to
    # pages of the response.
    #
    def list_by_database_as_lazy(location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      response = list_by_database_async(location_name, long_term_retention_server_name, long_term_retention_database_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_database_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Lists the long term retention backups for a given location.
    #
    # @param location_name [String] The location of the database
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] which provide lazy access to
    # pages of the response.
    #
    def list_by_location_as_lazy(location_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      response = list_by_location_async(location_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_location_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Lists the long term retention backups for a given server.
    #
    # @param location_name [String] The location of the database
    # @param long_term_retention_server_name [String] The name of the server
    # @param only_latest_per_database [Boolean] Whether or not to only get the
    # latest backup for each database.
    # @param database_state [LongTermRetentionDatabaseState] Whether to query
    # against just live databases, just deleted databases, or all databases.
    # Possible values include: 'All', 'Live', 'Deleted'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [LongTermRetentionBackupListResult] which provide lazy access to
    # pages of the response.
    #
    def list_by_server_as_lazy(location_name, long_term_retention_server_name, only_latest_per_database:nil, database_state:nil, custom_headers:nil)
      response = list_by_server_async(location_name, long_term_retention_server_name, only_latest_per_database:only_latest_per_database, database_state:database_state, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_server_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
