# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::SQL::Mgmt::V2017_10_01_preview
  #
  # The Azure SQL Database management API provides a RESTful set of web
  # services that interact with Azure SQL Database services to manage your
  # databases. The API enables you to create, retrieve, update, and delete
  # databases.
  #
  class ManagedDatabaseVulnerabilityAssessmentRuleBaselines
    include MsRestAzure

    #
    # Creates and initializes a new instance of the ManagedDatabaseVulnerabilityAssessmentRuleBaselines class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [SqlManagementClient] reference to the SqlManagementClient
    attr_reader :client

    #
    # Gets a database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DatabaseVulnerabilityAssessmentRuleBaseline] operation results.
    #
    def get(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:nil)
      response = get_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:nil)
      get_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:custom_headers).value!
    end

    #
    # Gets a database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'managed_instance_name is nil' if managed_instance_name.nil?
      fail ArgumentError, 'database_name is nil' if database_name.nil?
      vulnerability_assessment_name = 'default'
      fail ArgumentError, 'rule_id is nil' if rule_id.nil?
      fail ArgumentError, 'baseline_name is nil' if baseline_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/managedInstances/{managedInstanceName}/databases/{databaseName}/vulnerabilityAssessments/{vulnerabilityAssessmentName}/rules/{ruleId}/baselines/{baselineName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'managedInstanceName' => managed_instance_name,'databaseName' => database_name,'vulnerabilityAssessmentName' => vulnerability_assessment_name,'ruleId' => rule_id,'baselineName' => baseline_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_10_01_preview::Models::DatabaseVulnerabilityAssessmentRuleBaseline.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates or updates a database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param parameters [DatabaseVulnerabilityAssessmentRuleBaseline] The requested
    # rule baseline resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DatabaseVulnerabilityAssessmentRuleBaseline] operation results.
    #
    def create_or_update(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, parameters, custom_headers:nil)
      response = create_or_update_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates or updates a database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param parameters [DatabaseVulnerabilityAssessmentRuleBaseline] The requested
    # rule baseline resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_or_update_with_http_info(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, parameters, custom_headers:nil)
      create_or_update_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates or updates a database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param parameters [DatabaseVulnerabilityAssessmentRuleBaseline] The requested
    # rule baseline resource.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_or_update_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, parameters, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'managed_instance_name is nil' if managed_instance_name.nil?
      fail ArgumentError, 'database_name is nil' if database_name.nil?
      vulnerability_assessment_name = 'default'
      fail ArgumentError, 'rule_id is nil' if rule_id.nil?
      fail ArgumentError, 'baseline_name is nil' if baseline_name.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::SQL::Mgmt::V2017_10_01_preview::Models::DatabaseVulnerabilityAssessmentRuleBaseline.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/managedInstances/{managedInstanceName}/databases/{databaseName}/vulnerabilityAssessments/{vulnerabilityAssessmentName}/rules/{ruleId}/baselines/{baselineName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'managedInstanceName' => managed_instance_name,'databaseName' => database_name,'vulnerabilityAssessmentName' => vulnerability_assessment_name,'ruleId' => rule_id,'baselineName' => baseline_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::SQL::Mgmt::V2017_10_01_preview::Models::DatabaseVulnerabilityAssessmentRuleBaseline.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Removes the database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def delete(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:nil)
      response = delete_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Removes the database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def delete_with_http_info(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:nil)
      delete_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:custom_headers).value!
    end

    #
    # Removes the database's vulnerability assessment rule baseline.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the resource. You can obtain this value from the Azure Resource
    # Manager API or the portal.
    # @param managed_instance_name [String] The name of the managed instance.
    # @param database_name [String] The name of the database for which the
    # vulnerability assessment rule baseline is defined.
    # @param rule_id [String] The vulnerability assessment rule ID.
    # @param baseline_name [VulnerabilityAssessmentPolicyBaselineName] The name of
    # the vulnerability assessment rule baseline (default implies a baseline on a
    # database level rule and master for server level rule). Possible values
    # include: 'master', 'default'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def delete_async(resource_group_name, managed_instance_name, database_name, rule_id, baseline_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'managed_instance_name is nil' if managed_instance_name.nil?
      fail ArgumentError, 'database_name is nil' if database_name.nil?
      vulnerability_assessment_name = 'default'
      fail ArgumentError, 'rule_id is nil' if rule_id.nil?
      fail ArgumentError, 'baseline_name is nil' if baseline_name.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/managedInstances/{managedInstanceName}/databases/{databaseName}/vulnerabilityAssessments/{vulnerabilityAssessmentName}/rules/{ruleId}/baselines/{baselineName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'managedInstanceName' => managed_instance_name,'databaseName' => database_name,'vulnerabilityAssessmentName' => vulnerability_assessment_name,'ruleId' => rule_id,'baselineName' => baseline_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

  end
end
